import numpy as np
import pandas as pd
from mksc import reader

def load_data():
    """
    加载配置文件指定数据源，返回数据

    Returns:
        data: 配置文件数据框
    """
    cfg = reader.config()
    if bool(cfg.get('DATABASE', 'engine_url')):
        sql = cfg.get('DATABASE', 'sql')
        engine = cfg.get('DATABASE', 'engine_url')
        data = pd.read_sql(sql, engine)
    else:
        file = cfg.get('PATH', 'data_file')
        data = reader.file(file)

    # 大小写标准化
    for c in data.select_dtypes(include=['object']).columns:
        data[c] = data[c].str.lower()

    # 空值标准化
    data.replace("", np.nan, inplace=True)
    data.replace("null", np.nan, inplace=True)
    data.replace("none", np.nan, inplace=True)
    data.replace("na", np.nan, inplace=True)
    return data

def get_variable_type():
    """
    根据指定变量类型的配置表，返回各类型的变量列表

    Returns:
        numeric: 数值型变量列表
        category: 类别型变量列表
        datetime: 日期型变量列表
        label_name: 标签列
    """
    variable_type = pd.read_csv("conf/variable_type.csv", encoding='gbk')
    label_var = variable_type[variable_type.iloc[:, 2] == 'label']
    numeric_var = variable_type[(variable_type.iloc[:, 2] == 'numeric') & (variable_type.iloc[:, 1] == 1)]
    category_var = variable_type[(variable_type.iloc[:, 2] == 'category') & (variable_type.iloc[:, 1] == 1)]
    datetime_var = variable_type[(variable_type.iloc[:, 2] == 'datetime') & (variable_type.iloc[:, 1] == 1)]
    label_name = label_var['Variable'].tolist()[0]
    numeric = list(numeric_var['Variable'])
    category = list(category_var['Variable'])
    datetime = list(datetime_var['Variable'])
    return numeric, category, datetime, label_name
    
def variable_classify(feature):
    """
    对数据框feature的变量进行分类，返回各类别的变量列表

    Args:
        feature: 待分类的数据框

    Returns:
        numeric_var: 数值型变量列表
        category_var: 类别性变量列表
        datetime_var: 日期型变量列表
    """
    numeric_var = feature.select_dtypes(exclude=['object', 'datetime']).columns
    category_var = feature.select_dtypes('object').columns
    datetime_var = feature.select_dtypes('datetime').columns
    return numeric_var, category_var, datetime_var



from statsmodels.iolib.smpickle import load_pickle
from feature_engineering import preprocess
from feature_engineering import binning
from feature_engineering import scoring
import numpy as np
import pandas as pd
from .custom import Custom
import mksc

def transform(feature, feature_engineering):

    # 自定义特征组合模块
    feature = Custom.feature_combination(feature)

    # 极端值处理
    abnormal_value = feature_engineering['abnormal_value']
    for c in set(feature.columns) & set(abnormal_value['replace']):
        max_ = abnormal_value['result'][c]['max']
        min_ = abnormal_value['result'][c]['min']
        feature.loc[:, c] = feature.loc[:, c].apply(lambda x: x if (x < max_) & (x > min_) else np.nan)

    # 缺失值处理
    missing_filling = feature_engineering['missing_filling']
    for c in set(feature.columns) & set(missing_filling['replace']):
        fill_number = abnormal_value['result'][c]['fill_number']
        feature[c].fillna(fill_number, inplace=True)

    # woe转化
    woe_result = feature_engineering['woe_result']
    bin_result = feature_engineering['bin_result']
    feature = binning.woe_transform(feature, woe_result, bin_result)

    return feature

def main():

    # 数据、模型加载
    model = load_pickle('result/model.pickle')
    coefs = load_pickle('result/coefs.pickle')
    feature_engineering = load_pickle('result/feature_engineering.pickle')

    data = mksc.load_data()
    numeric_var, category_var, datetime_var, label_var = preprocess.get_variable_type()
    feature = data[numeric_var + category_var + datetime_var]
    label = None

    # 自定义数据清洗
    feature, label = Custom.clean_data(feature, label)

    # 数据类型转换
    feature[numeric_var] = feature[numeric_var].astype('float')
    feature[category_var] = feature[category_var].astype('object')
    feature[datetime_var] = feature[datetime_var].astype('datetime64')
    feature = data[coefs.keys]
    feature.drop(columns=['intercept_'], inplace=True)

    # 数据处理
    feature = transform(feature, feature_engineering)
    res = pd.DataFrame(model.predict(feature), columns=['label'])
    res = pd.concat([feature, res], axis=1)

    # 转化评分
    score_card = ''
    res = scoring.transform_score(res, score_card)
    # 结果保存
    res.save()


if __name__ == '__main__':
    main()



import pandas_profiling as pp
import pandas as pd
from statsmodels.iolib.smpickle import save_pickle
import mksc

def main():
    """
    探索性数据分析主程序入口
    生成结果报告、配置文件与样例数据
    """
    # 加载数据并保存本地
    data = mksc.load_data()
    save_pickle(data, 'data/data.pickle')

    # 保存分析报告
    sample = data.sample(min(len(data), 1000))
    sample.reset_index(drop=True, inplace=True)
    report = pp.ProfileReport(sample)
    report.to_file('result/eda_report.html')

    # 保存简单样本
    sample.to_excel('data/data_sample.xlsx', index=False)

    # 生成变量类别配置文件
    # 变量是否保留进行特征工程(isSave): 0-不保留；1-保留
    # 变量类型(Type): numeric-数值类型；category-类别类型；datetime-日期类型；label-标签列
    res = pd.DataFrame(zip(data.columns, [1]*len(data.columns), ['category']*len(data.columns), ['']*len(data.columns)),
                       columns=['Variable', 'isSave:[0/1]', 'Type:[numeric/category/datetime/label]', 'Comment'])
    res.to_csv("config/variable_type.csv", index=False)


if __name__ == '__main__':
    main()



import os
import sys
import socket
import logging
import logging.handlers
from mksc.core import reader

class Logger(object):
    """
    日志记录类
    """
    def __init__(self):
        self._setup_log_dir()
        self._setup_output_format()
        self._setup_file_handler()
        self._setup_stdout_handler()
        self._setup_loggers()

        self.info = self._logger_info.info
        self.warning = self._logger_warning.warning
        self.error = self._logger_error.error
        self.cfg = reader.config()

    def _setup_log_dir(self):
        """
        设置日志目录
        """
        self._log_dir = os.path.abspath(self.cfg.get('PATH', 'log_dir'))
        if os.path.exists(self._log_dir):
            pass
        else:
            os.mkdir(self._log_dir)
            print('[Logger]: created dir {}'.format(self._log_dir))

    def _setup_output_format(self):
        """
        设置日志格式
        """
        hostname = socket.gethostname()
        header = hostname + '\t' + '%(asctime)s' + '\t' + '%(levelname)s' + '\t' + '%(filename)s:%(lineno)s' + '\n'

        self._info_formatter = logging.Formatter(header + '- [INFO] ' + '%(message)s' + '\n')
        self._warning_formatter = logging.Formatter(header + '- [WARNING]' + '%(message)s' + '\n')
        self._error_formatter = logging.Formatter(header + '- [ERROR]' + '%(message)s' + '\n')

    def _setup_file_handler(self):
        """
        设置日志存储细节
        """
        n_bytes = 1024 * 1024 * 4  # 4MB / file
        n_backups = 10

        # 文件日志
        self._info_file_handler = logging.handlers.RotatingFileHandler(filename=os.path.join(self._log_dir, 'info.log'),
                                                                       mode='a',
                                                                       maxBytes=n_bytes,
                                                                       backupCount=n_backups)
        self._warning_file_handler = logging.handlers.RotatingFileHandler(os.path.join(self._log_dir, 'warning.log'),
                                                                          mode='a',
                                                                          maxBytes=n_bytes,
                                                                          backupCount=n_backups)
        self._error_file_handler = logging.handlers.RotatingFileHandler(os.path.join(self._log_dir, 'error.log'),
                                                                        mode='a',
                                                                        maxBytes=n_bytes,
                                                                        backupCount=n_backups)
        self._info_file_handler.setFormatter(self._info_formatter)
        self._warning_file_handler.setFormatter(self._warning_formatter)
        self._error_file_handler.setFormatter(self._error_formatter)

    def _setup_stdout_handler(self):
        """
        设置标准输出细节
        """
        self._stdout_handler = logging.StreamHandler(sys.stdout)
        self._stderr_handler = logging.StreamHandler(sys.stderr)

    def _setup_loggers(self):
        """
        获取logger实例
        指定日志的最低输出级别
        为logger添加的日志处理器
        """
        self._logger_info = logging.getLogger("info")
        self._logger_info.setLevel(logging.INFO)
        if not self._logger_info.handlers:
            self._logger_info.addHandler(self._stdout_handler)
            self._logger_info.addHandler(self._info_file_handler)

        self._logger_warning = logging.getLogger("warning")
        self._logger_warning.setLevel(logging.WARNING)
        if not self._logger_warning.handlers:
            self._logger_warning.addHandler(self._stdout_handler)
            self._logger_warning.addHandler(self._warning_file_handler)

        self._logger_error = logging.getLogger("error")
        self._logger_error.setLevel(logging.ERROR)
        if not self._logger_error.handlers:
            self._logger_error.addHandler(self._stderr_handler)
            self._logger_error.addHandler(self._error_file_handler)


# logger instance
logger = Logger()

if __name__ == '__main__':
    # test
    logger.info('test logger info')
    logger.warning('test logger warning')
    logger.error('test logger error')



from core.logger import logger
import time


def timing(func):
    """
    函数执行时间记录修饰函数
    """
    def wrapper(*args, **kwargs):
        logger.info('function started: {}'.format(func.__name__))
        t0 = time.time()
        result = func(*args, **kwargs)
        t1 = time.time()
        logger.info('function completed: {} ({:.3f}s)'.format(func.__name__, t1-t0))
        return result
    return wrapper



import pandas as pd

def woe_transform(feature, woe_result, bin_result):
    """
    将特征数据框转换为WOE分箱数据框

    Args:
        feature: 带转换数据框
        woe_result: 分组WOE值结果
        bin_result: 分组边界结果

    Returns:
        feature：已转换数据框
    """
    numeric_var = feature.select_dtypes(exclude=['object', 'datetime']).columns
    bin_var = list(set(numeric_var) & set(woe_result.keys()))
    for c in bin_var:
        woe = woe_result[c]
        feature[c] = pd.cut(feature[c], bin_result['result'][c])
        feature[c] = feature[c].apply(lambda x: x if pd.isna(x) else woe.loc[woe[c] == x, 'woe_i'].values[0])
        feature[c] = feature[c].astype('float')
        feature[c] = feature[c].apply(lambda x: woe.loc[0, 'woe_i'] if pd.isna(x) else x)
    return feature



from scipy.stats import boxcox

def fix_standard(feature):
    """
    对数据框中的数据进行正态化处理

    Args:
        feature: 待处理的数据框

    Returns:
        feature: 已处理数据框
        standard_lambda: 对应特征的lambda值
    """
    numeric_var = feature.select_dtypes(exclude=['object', 'datetime']).columns
    standard_lambda = {}
    for c in numeric_var:
        feature[c], lambda_ = boxcox(feature[c]+0.5)
        standard_lambda[c] = lambda_
    return feature, standard_lambda



import setuptools

with open("README.md", "r", encoding="utf_8_sig") as fh:
    long_description = fh.read()

setuptools.setup(
    name="mksc",
    version="1.0.9",
    author="wuhaoyu",
    author_email="wuhaoyu96@163.com",
    description="It is used to quickly build a scorecard project and dichotomy model package.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/HelloCoyen/mksc.git",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.4',
    entry_points={'console_scripts': ['mksc = mksc.cmd:main']},
    include_package_data=True
    )




def transform_score(data, score_card):
    base_score = score_card[score_card['Bins'] == '-']['Score'].values[0]
    data['Score'] = base_score
    for i in range(len(data)):
        score_i = base_score
        for k in set(score_card[score_card['Bins'] != '-']['Variables']):
            bin_score = score_card[(score_card['Woe'] == data.iloc[i][k]) & (score_card['Variables'] == k)]['Score']
            score_i += bin_score.values[0]
        data.iloc[i]['Score'] = score_i
    return data
